
%% Test of the C++ inverse dynamics generated by RobCoGen, for floating base
%% robots.
%%
%% Usage: [ME ROY] = testcpp_id_fb(MODEL, EXE)
%%
%% MODEL is the robot model (a data structure) in Roy's format.
%%
%% EXE is the full path of the program running the C++ RobCoGen inverse
%% dynamics for the same robot. The program must take the following command
%% line argument
%%   - status, velocity and desired acceleration for each joint
%%   - coordinates of the spatial velocity of the base (in base coordinates)
%%   - (optional) the full path with a text file with external forces
%%
%% That is, the program must be executable with something like the following:
%%    "<exe> <q'> <qd'> <qdd'> <base_v'> <path to fext>"
%%
%% See readme.md for more information and general documentation on the inputs
%% of this function.
%%
%% Author: Marco Frigerio
%
%%% This file is part of the Octave test functions for RobCoGen.
%%% Copyright © 2016 2017, Marco Frigerio (`marco.frigerio@iit.it`)
%%% See the LICENSE file for more information.

function [me roy] = testcpp_id_fb(roy_model, exe)

dof = length(roy_model.jtype) - 6;

q   = rand(dof,1);
qd  = rand(dof,1);
qdd = rand(dof,1);

base_v = rand(6,1);

%q   = zeros(dof,1);
%qd  = zeros(dof,1);
%qdd = zeros(dof,1);
%base_v = zeros(6,1);


Xfb = zeros(13,1);
Xfb(1) = 1;
Xfb(8:end,1) = base_v;


%
% Random external force vectors.
% (5 bodies are dummy links for the floating base virtual joint)
%%% ! WARNING ! %%% Check the readme for a note about external forces
%
fext = cell(1, roy_model.NB);
for i=1 : length(fext)
  fext{i} = rand(6, 1);
  %fext{i} = zeros(6, 1);
end

% Writes a file with the external forces, to be read by the C++ executable.
% Ignore the first 5 elements, as they refer to the dummy links of the floating base.
%
fid = fopen('fext.txt', 'w');
for i=6 : length(fext)
  fprintf(fid, "%f %f %f %f %f %f\n", fext{i});
end
fclose(fid);
fextFile = [pwd '/fext.txt']; % full absolute path of the external forces file


%
% Run the C++ executable, passing the various arguments
%
[foo output] = system( [exe ' ' num2str([q', qd', qdd', base_v']) ' ' fextFile ] );
output    = str2num(output);
me.base_a = output(1:6);
me.tau    = output(7:end);

%
% Execute Roy's inverse dynamics in Octave/Matlab
%
[roy.xdfb, roy.tau] = IDfb(roy_model, Xfb, q, qd, qdd, fext);

roy.xdfb( abs(roy.xdfb) < 1E-3 ) = 0;
roy.tau( abs(roy.tau) < 1E-3 ) = 0;
roy.base_a = roy.xdfb(8:end);

%
% Calculate and display the errors
%
err.base_a = roy.base_a - me.base_a;
err.tau    = roy.tau    - me.tau;
err.base_a( abs(err.base_a) < 1E-5 ) = 0;
err.tau   ( abs(err.tau) < 1E-5 ) = 0;

disp([roy.base_a, me.base_a, err.base_a]);
disp('');
disp([roy.tau   , me.tau   , err.tau   ]);


