
%% Test of the Octave inverse dynamics generated by RobCoGen, for floating base
%% robots.
%%
%% Usage: [ME ROY] = test_id_fb(ROY_MODEL, ROBCOGEN_MODEL)
%%
%% ROY_MODEL is the robot model (a data structure) in Roy's format.
%%
%% ROBCOGEN_MODEL is a structure with two fields, 'xm' and 'ip', respectively
%% the coordinate transforms for motion vectors and the inertia properties of
%% the links of the robot.
%%
%% See readme.md for more information and general documentation about the test
%% functions.
%%
%% Author: Marco Frigerio
%
%%% This file is part of the Octave test functions for RobCoGen.
%%% Copyright © 2016 2017, Marco Frigerio (`marco.frigerio@iit.it`)
%%% See the LICENSE file for more information.

function [me roy] = test_id_fb(roy_model, robcogen_model)

dof = length(roy_model.jtype) - 6;

q   = rand(dof,1);
qd  = rand(dof,1);
qdd = rand(dof,1);

% q   = zeros(dof,1);
% qd  = zeros(dof,1);
% qdd = zeros(dof,1);

base_v = rand(6,1);   % the current velocity of the base
%base_v = zeros(6,1);

% Lazy random quaternion:
quat = rand(4,1);
%quat = [1;0;0;0];
quat = quat ./ norm(quat);

% Rotation matrix according to the floating base orientation
world_R_base = quatToRotMx(quat);
base_R_world = world_R_base ';

% Gravity vector in floating base coordinates
gravity_base = base_R_world(:,3) * (-9.81);

% base->world rotation matrix for spatial vectors
world_RS_base = [world_R_base, zeros(3,3); zeros(3,3), world_R_base];

%
% RobCoGen Octave implementation of inverse dynamics
%
[params consts] = paramsAndConsts(robcogen_model);

robcogen_model.xm = updateMotionTransforms(robcogen_model.xm, q, params, consts);
[me.tau me.base_a] = inverseDynamics(robcogen_model.ip, robcogen_model.xm, base_v, [0;0;0;gravity_base], qd, qdd);

%
% Roy's routine. First prepare the inputs
%
Xfb = zeros(13,1);
Xfb(1:4) = quat;   % base orientation quaternion

% foating base velocity in world coordinates, as required by Roy's function
Xfb(8:end,1) = world_RS_base * base_v;

% Roy's default gravity vector is fine; [0;0; -9.81] in world coordinates

% Actual inverse dynamics
[roy.xdfb, roy.tau] = IDfb(roy_model, Xfb, q, qd, qdd);

% Rotate the spatial acceleration of the base in base coordinates, to compare it
% with RobCoGen
roy.base_a_base = world_RS_base' * roy.xdfb(8:end,1);

% Comparisons
%
err.tau    = roy.tau - me.tau;
err.base_a = roy.base_a_base - me.base_a;

err.tau( abs(err.tau) < 1E-5 ) = 0;
err.base_a( abs(err.base_a) < 1E-5 ) = 0;

display([roy.tau, me.tau, err.tau]);
display([roy.base_a_base, me.base_a, err.base_a]);




function world_R_base = quatToRotMx( quat )

w = quat(1);
x = quat(2);
y = quat(3);
z = quat(4);

world_R_base = [...
    1 - 2*z*z - 2*y*y ,   2*x*y - 2*w*z   ,   2*x*z + 2*w*y ;
    2*w*z + 2*x*y     , 1 - 2*z*z - 2*x*x ,   2*y*z - 2*w*x  ;
    2*x*z - 2*w*y     , 2*y*z + 2*w*x     , 1 - 2*y*y - 2*x*x ];


