
%% Test of the Octave function generated by RobCoGen to compute the Joint Space
%% Inertia Matrix (JSIM), for fixed base robots.
%%
%% Usage: [ME ROY] = test_jsim(ROY_MODEL, RCG)
%%
%% ROY_MODEL is the robot model (a data structure) in Roy's format.
%%
%% RCG is a structure with two fields, 'xf' and 'ip', respectively
%% the coordinate transforms for force vectors and the inertia properties of
%% the links of the robot.
%%
%% See readme.md for more information and general documentation about the test
%% functions.
%%
%% Author: Marco Frigerio
%
%%% This file is part of the Octave test functions for RobCoGen.
%%% Copyright © 2016 2017, Marco Frigerio (`marco.frigerio@iit.it`)
%%% See the LICENSE file for more information.

function [me roy] = test_jsim(roy_model, rcg)


dof = length(roy_model.jtype);
q   = rand(dof,1) * 2;
qd  = rand(dof,1) * 2;

% RobCoGen implementation:
[params consts] = paramsAndConsts(rcg);

rcg.xf = updateForceTransforms(rcg.xf, q, params, consts);
me.H   = updateJSIM(rcg.ip, rcg.xf);

% Roy's implementation
[roy.H, roy.C] = HandC( roy_model, q, qd ); % Featherstone's routine

err = roy.H - me.H;
err( abs(err) < 1E-3 ) = 0;

disp(err);


