#ifndef IIT_ROBOGEN__FANCY_TRAITS_H_
#define IIT_ROBOGEN__FANCY_TRAITS_H_

#include "declarations.h"
#include "transforms.h"
#include "inverse_dynamics.h"
#include "forward_dynamics.h"
#include "jsim.h"
#include "inertia_properties.h"

namespace iit {
namespace Fancy {

struct Traits {
    typedef typename Fancy::ScalarTraits ScalarTraits;

    typedef typename Fancy::JointState JointState;

    typedef typename Fancy::JointIdentifiers JointID;
    typedef typename Fancy::LinkIdentifiers  LinkID;

    typedef typename Fancy::HomogeneousTransforms HomogeneousTransforms;
    typedef typename Fancy::MotionTransforms MotionTransforms;
    typedef typename Fancy::ForceTransforms ForceTransforms;

    typedef typename Fancy::dyn::InertiaProperties InertiaProperties;
    typedef typename Fancy::dyn::ForwardDynamics FwdDynEngine;
    typedef typename Fancy::dyn::InverseDynamics InvDynEngine;
    typedef typename Fancy::dyn::JSIM JSIM;

    static const int joints_count = Fancy::jointsCount;
    static const int links_count  = Fancy::linksCount;
    static const bool floating_base = false;

    static inline const JointID* orderedJointIDs();
    static inline const LinkID*  orderedLinkIDs();
};


inline const Traits::JointID*  Traits::orderedJointIDs() {
    return Fancy::orderedJointIDs;
}
inline const Traits::LinkID*  Traits::orderedLinkIDs() {
    return Fancy::orderedLinkIDs;
}

}
}

#endif
