
%% A structure with the Fancy robot model in the format described
%% in "A beginner's guide to 6-D vectors (part 2)" by Roy Featherstone.
%%
%% FANCY = FANCY(consts, params)
%%
%% FANCY is the data struct with the robot model
%% parameters.
%% Depending on the original robot model in the Kinematics-DSL format,
%% and on configuration options, this function may take up to two
%% arguments:
%%
%% CONSTS is a struct with all the constant properties
%% of the robot model; it is required if constant-folding was disabled
%% during code generation. Otherwise, the constant numerical properties
%% of the model appear explicitly as float literals.
%%
%% PARAMS is a struct with the current value of the
%% possibly varying model properties, i.e. the model parameters. This
%% argument is required if the original robot model is parametric. If
%% not, ignore the argument.
%%
%% This file has been automatically generated by RobCoGen.

function fancy = Fancy(consts, params)

fancy.robotname = 'Fancy';

fancy.NB = 5;
fancy.parent = zeros(1, 5);

fancy.parent(1) = 0;
fancy.parent(2) = 1;
fancy.parent(3) = 2;
fancy.parent(4) = 3;
fancy.parent(5) = 4;

fancy.jtype{1} =  'Rz' ;
fancy.jtype{2} =  'Pz' ;
fancy.jtype{3} =  'Rz' ;
fancy.jtype{4} =  'Pz' ;
fancy.jtype{5} =  'Rz' ;

fancy.Xtree{1} = rotz(0.0) * roty(0.0) * rotx(0.0) * ...
          xlt([0.0 0.0 0.0]);
fancy.Xtree{2} = rotz(0.0) * roty(0.0) * rotx(-1.5707963705062866) * ...
          xlt([consts.tx_jB 0.0 0.0]);
fancy.Xtree{3} = rotz(0.0) * roty(0.0) * rotx(0.0) * ...
          xlt([0.0 0.0 consts.tz_jC]);
fancy.Xtree{4} = rotz(0.0) * roty(0.0) * rotx(1.5707963705062866) * ...
          xlt([consts.tx_jD 0.0 0.0]);
fancy.Xtree{5} = rotz(0.0) * roty(0.0) * rotx(0.0) * ...
          xlt([0.0 0.0 consts.tz_jE]);


tmpI = ...
[[ consts.ix_link1  ,	-(0.0),	-(0.0)];
[-(0.0),	  consts.iy_link1  ,	-(0.0)];
[-(0.0),	-(0.0),	  consts.iz_link1]];
tmpC = skew([consts.comx_link1; 0.0; 0.0]);
tmpM = consts.m_link1;
fancy.I{1} = [tmpI, tmpM *tmpC; tmpM *tmpC', eye(3)*tmpM ];


tmpI = ...
[[ consts.ix_link2  ,	-(0.0),	-(0.0)];
[-(0.0),	  consts.iy_link2  ,	-(0.0)];
[-(0.0),	-(0.0),	  consts.iz_link2]];
tmpC = skew([0.0; 0.0; consts.comz_link2]);
tmpM = consts.m_link2;
fancy.I{2} = [tmpI, tmpM *tmpC; tmpM *tmpC', eye(3)*tmpM ];


tmpI = ...
[[ consts.ix_link3  ,	-(0.0),	-(0.0)];
[-(0.0),	  consts.iy_link3  ,	-(0.0)];
[-(0.0),	-(0.0),	  consts.iz_link3]];
tmpC = skew([consts.comx_link3; 0.0; 0.0]);
tmpM = consts.m_link3;
fancy.I{3} = [tmpI, tmpM *tmpC; tmpM *tmpC', eye(3)*tmpM ];


tmpI = ...
[[ consts.ix_link4  ,	-(0.0),	-(0.0)];
[-(0.0),	  consts.iy_link4  ,	-(0.0)];
[-(0.0),	-(0.0),	  consts.iz_link4]];
tmpC = skew([0.0; 0.0; consts.comz_link4]);
tmpM = consts.m_link4;
fancy.I{4} = [tmpI, tmpM *tmpC; tmpM *tmpC', eye(3)*tmpM ];


tmpI = ...
[[ consts.ix_link5  ,	-(0.0),	-(0.0)];
[-(0.0),	  consts.iy_link5  ,	-(0.0)];
[-(0.0),	-(0.0),	  consts.iz_link5]];
tmpC = skew([consts.comx_link5; 0.0; 0.0]);
tmpM = consts.m_link5;
fancy.I{5} = [tmpI, tmpM *tmpC; tmpM *tmpC', eye(3)*tmpM ];

