%% Inertia properties expressed in the default link-frame, for each
%% link of the robot Fancy.
%%
%% OUT = INERTIAPROPERTIES(CONSTS, PARAMS)
%%
%% OUT is a structure with a field for each link of the robot, each
%% field being in turn a struct with the inertia properties.
%%
%% Depending on the original robot model and on the code generator
%% configuration, this function may take up to two arguments:
%%
%% CONSTS is a struct with all the constant properties
%% of the robot model; it is required if constant-folding was disabled
%% during code generation. Otherwise, the constant numerical properties
%% of the model appear explicitly as float literals.
%%
%% PARAMS is a struct with the current value of the
%% possibly varying model properties, i.e. the model parameters. This
%% argument is required if the original robot model is parametric. If
%% not, ignore the argument.
%%
%% This file has been automatically generated by RobCoGen.

function out = inertiaProperties(consts, params)

out.lf_base0.mass   = consts.m_base0;
out.lf_base0.com    = [0.0; 0.0; 0.0];
out.lf_base0.tensor = ...
    [[ 0.0  ,	-(0.0),	-(0.0)];
    [-(0.0),	  0.0  ,	-(0.0)];
    [-(0.0),	-(0.0),	  0.0]];

com = out.lf_base0.com;
block = [  0,    -com(3),  com(2);
         com(3),  0,      -com(1);
        -com(2),  com(1),  0 ] * out.lf_base0.mass;
out.lf_base0.tensor6D = [out.lf_base0.tensor, block; block', out.lf_base0.mass*eye(3)];


out.lf_link1.mass   = consts.m_link1;
out.lf_link1.com    = [consts.comx_link1; 0.0; 0.0];
out.lf_link1.tensor = ...
    [[ consts.ix_link1  ,	-(0.0),	-(0.0)];
    [-(0.0),	  consts.iy_link1  ,	-(0.0)];
    [-(0.0),	-(0.0),	  consts.iz_link1]];

com = out.lf_link1.com;
block = [  0,    -com(3),  com(2);
         com(3),  0,      -com(1);
        -com(2),  com(1),  0 ] * out.lf_link1.mass;
out.lf_link1.tensor6D = [out.lf_link1.tensor, block; block', out.lf_link1.mass*eye(3)];


out.lf_link2.mass   = consts.m_link2;
out.lf_link2.com    = [0.0; 0.0; consts.comz_link2];
out.lf_link2.tensor = ...
    [[ consts.ix_link2  ,	-(0.0),	-(0.0)];
    [-(0.0),	  consts.iy_link2  ,	-(0.0)];
    [-(0.0),	-(0.0),	  consts.iz_link2]];

com = out.lf_link2.com;
block = [  0,    -com(3),  com(2);
         com(3),  0,      -com(1);
        -com(2),  com(1),  0 ] * out.lf_link2.mass;
out.lf_link2.tensor6D = [out.lf_link2.tensor, block; block', out.lf_link2.mass*eye(3)];


out.lf_link3.mass   = consts.m_link3;
out.lf_link3.com    = [consts.comx_link3; 0.0; 0.0];
out.lf_link3.tensor = ...
    [[ consts.ix_link3  ,	-(0.0),	-(0.0)];
    [-(0.0),	  consts.iy_link3  ,	-(0.0)];
    [-(0.0),	-(0.0),	  consts.iz_link3]];

com = out.lf_link3.com;
block = [  0,    -com(3),  com(2);
         com(3),  0,      -com(1);
        -com(2),  com(1),  0 ] * out.lf_link3.mass;
out.lf_link3.tensor6D = [out.lf_link3.tensor, block; block', out.lf_link3.mass*eye(3)];


out.lf_link4.mass   = consts.m_link4;
out.lf_link4.com    = [0.0; 0.0; consts.comz_link4];
out.lf_link4.tensor = ...
    [[ consts.ix_link4  ,	-(0.0),	-(0.0)];
    [-(0.0),	  consts.iy_link4  ,	-(0.0)];
    [-(0.0),	-(0.0),	  consts.iz_link4]];

com = out.lf_link4.com;
block = [  0,    -com(3),  com(2);
         com(3),  0,      -com(1);
        -com(2),  com(1),  0 ] * out.lf_link4.mass;
out.lf_link4.tensor6D = [out.lf_link4.tensor, block; block', out.lf_link4.mass*eye(3)];


out.lf_link5.mass   = consts.m_link5;
out.lf_link5.com    = [consts.comx_link5; 0.0; 0.0];
out.lf_link5.tensor = ...
    [[ consts.ix_link5  ,	-(0.0),	-(0.0)];
    [-(0.0),	  consts.iy_link5  ,	-(0.0)];
    [-(0.0),	-(0.0),	  consts.iz_link5]];

com = out.lf_link5.com;
block = [  0,    -com(3),  com(2);
         com(3),  0,      -com(1);
        -com(2),  com(1),  0 ] * out.lf_link5.mass;
out.lf_link5.tensor6D = [out.lf_link5.tensor, block; block', out.lf_link5.mass*eye(3)];


