#ifndef HYQ_TRANSFORMS_H_
#define HYQ_TRANSFORMS_H_

#include <iit/rbd/TransformsBase.h>
#include "declarations.h"
#include "model_constants.h"
#include "kinematics_parameters.h"

namespace iit {
namespace HyQ {

struct Parameters
{
    struct AngleFuncValues {
        AngleFuncValues() {
            update();
        }

        void update()
        {
        }
    };

    Params_lengths lengths;
    Params_angles angles;
    AngleFuncValues trig = AngleFuncValues();
};

// The type of the "vector" with the status of the variables
typedef JointState state_t;

template<class M>
using TransformMotion = iit::rbd::SpatialTransformBase<state_t, M>;

template<class M>
using TransformForce = iit::rbd::SpatialTransformBase<state_t, M>;

template<class M>
using TransformHomogeneous = iit::rbd::HomogeneousTransformBase<state_t, M>;

/**
 * The class for the 6-by-6 coordinates transformation matrices for
 * spatial motion vectors.
 */
class MotionTransforms
{
public:
    class Dummy {};
    typedef TransformMotion<Dummy>::MatrixType MatrixType;

    struct Type_fr_LF_hipassembly_X_fr_trunk : public TransformMotion<Type_fr_LF_hipassembly_X_fr_trunk>
    {
        Type_fr_LF_hipassembly_X_fr_trunk();
        const Type_fr_LF_hipassembly_X_fr_trunk& update(const state_t&);
    };
    
    struct Type_fr_trunk_X_fr_LF_hipassembly : public TransformMotion<Type_fr_trunk_X_fr_LF_hipassembly>
    {
        Type_fr_trunk_X_fr_LF_hipassembly();
        const Type_fr_trunk_X_fr_LF_hipassembly& update(const state_t&);
    };
    
    struct Type_fr_LF_upperleg_X_fr_LF_hipassembly : public TransformMotion<Type_fr_LF_upperleg_X_fr_LF_hipassembly>
    {
        Type_fr_LF_upperleg_X_fr_LF_hipassembly();
        const Type_fr_LF_upperleg_X_fr_LF_hipassembly& update(const state_t&);
    };
    
    struct Type_fr_LF_hipassembly_X_fr_LF_upperleg : public TransformMotion<Type_fr_LF_hipassembly_X_fr_LF_upperleg>
    {
        Type_fr_LF_hipassembly_X_fr_LF_upperleg();
        const Type_fr_LF_hipassembly_X_fr_LF_upperleg& update(const state_t&);
    };
    
    struct Type_fr_LF_lowerleg_X_fr_LF_upperleg : public TransformMotion<Type_fr_LF_lowerleg_X_fr_LF_upperleg>
    {
        Type_fr_LF_lowerleg_X_fr_LF_upperleg();
        const Type_fr_LF_lowerleg_X_fr_LF_upperleg& update(const state_t&);
    };
    
    struct Type_fr_LF_upperleg_X_fr_LF_lowerleg : public TransformMotion<Type_fr_LF_upperleg_X_fr_LF_lowerleg>
    {
        Type_fr_LF_upperleg_X_fr_LF_lowerleg();
        const Type_fr_LF_upperleg_X_fr_LF_lowerleg& update(const state_t&);
    };
    
    struct Type_fr_RF_hipassembly_X_fr_trunk : public TransformMotion<Type_fr_RF_hipassembly_X_fr_trunk>
    {
        Type_fr_RF_hipassembly_X_fr_trunk();
        const Type_fr_RF_hipassembly_X_fr_trunk& update(const state_t&);
    };
    
    struct Type_fr_trunk_X_fr_RF_hipassembly : public TransformMotion<Type_fr_trunk_X_fr_RF_hipassembly>
    {
        Type_fr_trunk_X_fr_RF_hipassembly();
        const Type_fr_trunk_X_fr_RF_hipassembly& update(const state_t&);
    };
    
    struct Type_fr_RF_upperleg_X_fr_RF_hipassembly : public TransformMotion<Type_fr_RF_upperleg_X_fr_RF_hipassembly>
    {
        Type_fr_RF_upperleg_X_fr_RF_hipassembly();
        const Type_fr_RF_upperleg_X_fr_RF_hipassembly& update(const state_t&);
    };
    
    struct Type_fr_RF_hipassembly_X_fr_RF_upperleg : public TransformMotion<Type_fr_RF_hipassembly_X_fr_RF_upperleg>
    {
        Type_fr_RF_hipassembly_X_fr_RF_upperleg();
        const Type_fr_RF_hipassembly_X_fr_RF_upperleg& update(const state_t&);
    };
    
    struct Type_fr_RF_lowerleg_X_fr_RF_upperleg : public TransformMotion<Type_fr_RF_lowerleg_X_fr_RF_upperleg>
    {
        Type_fr_RF_lowerleg_X_fr_RF_upperleg();
        const Type_fr_RF_lowerleg_X_fr_RF_upperleg& update(const state_t&);
    };
    
    struct Type_fr_RF_upperleg_X_fr_RF_lowerleg : public TransformMotion<Type_fr_RF_upperleg_X_fr_RF_lowerleg>
    {
        Type_fr_RF_upperleg_X_fr_RF_lowerleg();
        const Type_fr_RF_upperleg_X_fr_RF_lowerleg& update(const state_t&);
    };
    
    struct Type_fr_LH_hipassembly_X_fr_trunk : public TransformMotion<Type_fr_LH_hipassembly_X_fr_trunk>
    {
        Type_fr_LH_hipassembly_X_fr_trunk();
        const Type_fr_LH_hipassembly_X_fr_trunk& update(const state_t&);
    };
    
    struct Type_fr_trunk_X_fr_LH_hipassembly : public TransformMotion<Type_fr_trunk_X_fr_LH_hipassembly>
    {
        Type_fr_trunk_X_fr_LH_hipassembly();
        const Type_fr_trunk_X_fr_LH_hipassembly& update(const state_t&);
    };
    
    struct Type_fr_LH_upperleg_X_fr_LH_hipassembly : public TransformMotion<Type_fr_LH_upperleg_X_fr_LH_hipassembly>
    {
        Type_fr_LH_upperleg_X_fr_LH_hipassembly();
        const Type_fr_LH_upperleg_X_fr_LH_hipassembly& update(const state_t&);
    };
    
    struct Type_fr_LH_hipassembly_X_fr_LH_upperleg : public TransformMotion<Type_fr_LH_hipassembly_X_fr_LH_upperleg>
    {
        Type_fr_LH_hipassembly_X_fr_LH_upperleg();
        const Type_fr_LH_hipassembly_X_fr_LH_upperleg& update(const state_t&);
    };
    
    struct Type_fr_LH_lowerleg_X_fr_LH_upperleg : public TransformMotion<Type_fr_LH_lowerleg_X_fr_LH_upperleg>
    {
        Type_fr_LH_lowerleg_X_fr_LH_upperleg();
        const Type_fr_LH_lowerleg_X_fr_LH_upperleg& update(const state_t&);
    };
    
    struct Type_fr_LH_upperleg_X_fr_LH_lowerleg : public TransformMotion<Type_fr_LH_upperleg_X_fr_LH_lowerleg>
    {
        Type_fr_LH_upperleg_X_fr_LH_lowerleg();
        const Type_fr_LH_upperleg_X_fr_LH_lowerleg& update(const state_t&);
    };
    
    struct Type_fr_RH_hipassembly_X_fr_trunk : public TransformMotion<Type_fr_RH_hipassembly_X_fr_trunk>
    {
        Type_fr_RH_hipassembly_X_fr_trunk();
        const Type_fr_RH_hipassembly_X_fr_trunk& update(const state_t&);
    };
    
    struct Type_fr_trunk_X_fr_RH_hipassembly : public TransformMotion<Type_fr_trunk_X_fr_RH_hipassembly>
    {
        Type_fr_trunk_X_fr_RH_hipassembly();
        const Type_fr_trunk_X_fr_RH_hipassembly& update(const state_t&);
    };
    
    struct Type_fr_RH_upperleg_X_fr_RH_hipassembly : public TransformMotion<Type_fr_RH_upperleg_X_fr_RH_hipassembly>
    {
        Type_fr_RH_upperleg_X_fr_RH_hipassembly();
        const Type_fr_RH_upperleg_X_fr_RH_hipassembly& update(const state_t&);
    };
    
    struct Type_fr_RH_hipassembly_X_fr_RH_upperleg : public TransformMotion<Type_fr_RH_hipassembly_X_fr_RH_upperleg>
    {
        Type_fr_RH_hipassembly_X_fr_RH_upperleg();
        const Type_fr_RH_hipassembly_X_fr_RH_upperleg& update(const state_t&);
    };
    
    struct Type_fr_RH_lowerleg_X_fr_RH_upperleg : public TransformMotion<Type_fr_RH_lowerleg_X_fr_RH_upperleg>
    {
        Type_fr_RH_lowerleg_X_fr_RH_upperleg();
        const Type_fr_RH_lowerleg_X_fr_RH_upperleg& update(const state_t&);
    };
    
    struct Type_fr_RH_upperleg_X_fr_RH_lowerleg : public TransformMotion<Type_fr_RH_upperleg_X_fr_RH_lowerleg>
    {
        Type_fr_RH_upperleg_X_fr_RH_lowerleg();
        const Type_fr_RH_upperleg_X_fr_RH_lowerleg& update(const state_t&);
    };
    
public:
    MotionTransforms();
    void updateParams(const Params_lengths&, const Params_angles&);

    Type_fr_LF_hipassembly_X_fr_trunk fr_LF_hipassembly_X_fr_trunk;
    Type_fr_trunk_X_fr_LF_hipassembly fr_trunk_X_fr_LF_hipassembly;
    Type_fr_LF_upperleg_X_fr_LF_hipassembly fr_LF_upperleg_X_fr_LF_hipassembly;
    Type_fr_LF_hipassembly_X_fr_LF_upperleg fr_LF_hipassembly_X_fr_LF_upperleg;
    Type_fr_LF_lowerleg_X_fr_LF_upperleg fr_LF_lowerleg_X_fr_LF_upperleg;
    Type_fr_LF_upperleg_X_fr_LF_lowerleg fr_LF_upperleg_X_fr_LF_lowerleg;
    Type_fr_RF_hipassembly_X_fr_trunk fr_RF_hipassembly_X_fr_trunk;
    Type_fr_trunk_X_fr_RF_hipassembly fr_trunk_X_fr_RF_hipassembly;
    Type_fr_RF_upperleg_X_fr_RF_hipassembly fr_RF_upperleg_X_fr_RF_hipassembly;
    Type_fr_RF_hipassembly_X_fr_RF_upperleg fr_RF_hipassembly_X_fr_RF_upperleg;
    Type_fr_RF_lowerleg_X_fr_RF_upperleg fr_RF_lowerleg_X_fr_RF_upperleg;
    Type_fr_RF_upperleg_X_fr_RF_lowerleg fr_RF_upperleg_X_fr_RF_lowerleg;
    Type_fr_LH_hipassembly_X_fr_trunk fr_LH_hipassembly_X_fr_trunk;
    Type_fr_trunk_X_fr_LH_hipassembly fr_trunk_X_fr_LH_hipassembly;
    Type_fr_LH_upperleg_X_fr_LH_hipassembly fr_LH_upperleg_X_fr_LH_hipassembly;
    Type_fr_LH_hipassembly_X_fr_LH_upperleg fr_LH_hipassembly_X_fr_LH_upperleg;
    Type_fr_LH_lowerleg_X_fr_LH_upperleg fr_LH_lowerleg_X_fr_LH_upperleg;
    Type_fr_LH_upperleg_X_fr_LH_lowerleg fr_LH_upperleg_X_fr_LH_lowerleg;
    Type_fr_RH_hipassembly_X_fr_trunk fr_RH_hipassembly_X_fr_trunk;
    Type_fr_trunk_X_fr_RH_hipassembly fr_trunk_X_fr_RH_hipassembly;
    Type_fr_RH_upperleg_X_fr_RH_hipassembly fr_RH_upperleg_X_fr_RH_hipassembly;
    Type_fr_RH_hipassembly_X_fr_RH_upperleg fr_RH_hipassembly_X_fr_RH_upperleg;
    Type_fr_RH_lowerleg_X_fr_RH_upperleg fr_RH_lowerleg_X_fr_RH_upperleg;
    Type_fr_RH_upperleg_X_fr_RH_lowerleg fr_RH_upperleg_X_fr_RH_lowerleg;

protected:
    Parameters params;

}; //class 'MotionTransforms'

/**
 * The class for the 6-by-6 coordinates transformation matrices for
 * spatial force vectors.
 */
class ForceTransforms
{
public:
    class Dummy {};
    typedef TransformForce<Dummy>::MatrixType MatrixType;

    struct Type_fr_LF_hipassembly_X_fr_trunk : public TransformForce<Type_fr_LF_hipassembly_X_fr_trunk>
    {
        Type_fr_LF_hipassembly_X_fr_trunk();
        const Type_fr_LF_hipassembly_X_fr_trunk& update(const state_t&);
    };
    
    struct Type_fr_trunk_X_fr_LF_hipassembly : public TransformForce<Type_fr_trunk_X_fr_LF_hipassembly>
    {
        Type_fr_trunk_X_fr_LF_hipassembly();
        const Type_fr_trunk_X_fr_LF_hipassembly& update(const state_t&);
    };
    
    struct Type_fr_LF_upperleg_X_fr_LF_hipassembly : public TransformForce<Type_fr_LF_upperleg_X_fr_LF_hipassembly>
    {
        Type_fr_LF_upperleg_X_fr_LF_hipassembly();
        const Type_fr_LF_upperleg_X_fr_LF_hipassembly& update(const state_t&);
    };
    
    struct Type_fr_LF_hipassembly_X_fr_LF_upperleg : public TransformForce<Type_fr_LF_hipassembly_X_fr_LF_upperleg>
    {
        Type_fr_LF_hipassembly_X_fr_LF_upperleg();
        const Type_fr_LF_hipassembly_X_fr_LF_upperleg& update(const state_t&);
    };
    
    struct Type_fr_LF_lowerleg_X_fr_LF_upperleg : public TransformForce<Type_fr_LF_lowerleg_X_fr_LF_upperleg>
    {
        Type_fr_LF_lowerleg_X_fr_LF_upperleg();
        const Type_fr_LF_lowerleg_X_fr_LF_upperleg& update(const state_t&);
    };
    
    struct Type_fr_LF_upperleg_X_fr_LF_lowerleg : public TransformForce<Type_fr_LF_upperleg_X_fr_LF_lowerleg>
    {
        Type_fr_LF_upperleg_X_fr_LF_lowerleg();
        const Type_fr_LF_upperleg_X_fr_LF_lowerleg& update(const state_t&);
    };
    
    struct Type_fr_RF_hipassembly_X_fr_trunk : public TransformForce<Type_fr_RF_hipassembly_X_fr_trunk>
    {
        Type_fr_RF_hipassembly_X_fr_trunk();
        const Type_fr_RF_hipassembly_X_fr_trunk& update(const state_t&);
    };
    
    struct Type_fr_trunk_X_fr_RF_hipassembly : public TransformForce<Type_fr_trunk_X_fr_RF_hipassembly>
    {
        Type_fr_trunk_X_fr_RF_hipassembly();
        const Type_fr_trunk_X_fr_RF_hipassembly& update(const state_t&);
    };
    
    struct Type_fr_RF_upperleg_X_fr_RF_hipassembly : public TransformForce<Type_fr_RF_upperleg_X_fr_RF_hipassembly>
    {
        Type_fr_RF_upperleg_X_fr_RF_hipassembly();
        const Type_fr_RF_upperleg_X_fr_RF_hipassembly& update(const state_t&);
    };
    
    struct Type_fr_RF_hipassembly_X_fr_RF_upperleg : public TransformForce<Type_fr_RF_hipassembly_X_fr_RF_upperleg>
    {
        Type_fr_RF_hipassembly_X_fr_RF_upperleg();
        const Type_fr_RF_hipassembly_X_fr_RF_upperleg& update(const state_t&);
    };
    
    struct Type_fr_RF_lowerleg_X_fr_RF_upperleg : public TransformForce<Type_fr_RF_lowerleg_X_fr_RF_upperleg>
    {
        Type_fr_RF_lowerleg_X_fr_RF_upperleg();
        const Type_fr_RF_lowerleg_X_fr_RF_upperleg& update(const state_t&);
    };
    
    struct Type_fr_RF_upperleg_X_fr_RF_lowerleg : public TransformForce<Type_fr_RF_upperleg_X_fr_RF_lowerleg>
    {
        Type_fr_RF_upperleg_X_fr_RF_lowerleg();
        const Type_fr_RF_upperleg_X_fr_RF_lowerleg& update(const state_t&);
    };
    
    struct Type_fr_LH_hipassembly_X_fr_trunk : public TransformForce<Type_fr_LH_hipassembly_X_fr_trunk>
    {
        Type_fr_LH_hipassembly_X_fr_trunk();
        const Type_fr_LH_hipassembly_X_fr_trunk& update(const state_t&);
    };
    
    struct Type_fr_trunk_X_fr_LH_hipassembly : public TransformForce<Type_fr_trunk_X_fr_LH_hipassembly>
    {
        Type_fr_trunk_X_fr_LH_hipassembly();
        const Type_fr_trunk_X_fr_LH_hipassembly& update(const state_t&);
    };
    
    struct Type_fr_LH_upperleg_X_fr_LH_hipassembly : public TransformForce<Type_fr_LH_upperleg_X_fr_LH_hipassembly>
    {
        Type_fr_LH_upperleg_X_fr_LH_hipassembly();
        const Type_fr_LH_upperleg_X_fr_LH_hipassembly& update(const state_t&);
    };
    
    struct Type_fr_LH_hipassembly_X_fr_LH_upperleg : public TransformForce<Type_fr_LH_hipassembly_X_fr_LH_upperleg>
    {
        Type_fr_LH_hipassembly_X_fr_LH_upperleg();
        const Type_fr_LH_hipassembly_X_fr_LH_upperleg& update(const state_t&);
    };
    
    struct Type_fr_LH_lowerleg_X_fr_LH_upperleg : public TransformForce<Type_fr_LH_lowerleg_X_fr_LH_upperleg>
    {
        Type_fr_LH_lowerleg_X_fr_LH_upperleg();
        const Type_fr_LH_lowerleg_X_fr_LH_upperleg& update(const state_t&);
    };
    
    struct Type_fr_LH_upperleg_X_fr_LH_lowerleg : public TransformForce<Type_fr_LH_upperleg_X_fr_LH_lowerleg>
    {
        Type_fr_LH_upperleg_X_fr_LH_lowerleg();
        const Type_fr_LH_upperleg_X_fr_LH_lowerleg& update(const state_t&);
    };
    
    struct Type_fr_RH_hipassembly_X_fr_trunk : public TransformForce<Type_fr_RH_hipassembly_X_fr_trunk>
    {
        Type_fr_RH_hipassembly_X_fr_trunk();
        const Type_fr_RH_hipassembly_X_fr_trunk& update(const state_t&);
    };
    
    struct Type_fr_trunk_X_fr_RH_hipassembly : public TransformForce<Type_fr_trunk_X_fr_RH_hipassembly>
    {
        Type_fr_trunk_X_fr_RH_hipassembly();
        const Type_fr_trunk_X_fr_RH_hipassembly& update(const state_t&);
    };
    
    struct Type_fr_RH_upperleg_X_fr_RH_hipassembly : public TransformForce<Type_fr_RH_upperleg_X_fr_RH_hipassembly>
    {
        Type_fr_RH_upperleg_X_fr_RH_hipassembly();
        const Type_fr_RH_upperleg_X_fr_RH_hipassembly& update(const state_t&);
    };
    
    struct Type_fr_RH_hipassembly_X_fr_RH_upperleg : public TransformForce<Type_fr_RH_hipassembly_X_fr_RH_upperleg>
    {
        Type_fr_RH_hipassembly_X_fr_RH_upperleg();
        const Type_fr_RH_hipassembly_X_fr_RH_upperleg& update(const state_t&);
    };
    
    struct Type_fr_RH_lowerleg_X_fr_RH_upperleg : public TransformForce<Type_fr_RH_lowerleg_X_fr_RH_upperleg>
    {
        Type_fr_RH_lowerleg_X_fr_RH_upperleg();
        const Type_fr_RH_lowerleg_X_fr_RH_upperleg& update(const state_t&);
    };
    
    struct Type_fr_RH_upperleg_X_fr_RH_lowerleg : public TransformForce<Type_fr_RH_upperleg_X_fr_RH_lowerleg>
    {
        Type_fr_RH_upperleg_X_fr_RH_lowerleg();
        const Type_fr_RH_upperleg_X_fr_RH_lowerleg& update(const state_t&);
    };
    
public:
    ForceTransforms();
    void updateParams(const Params_lengths&, const Params_angles&);

    Type_fr_LF_hipassembly_X_fr_trunk fr_LF_hipassembly_X_fr_trunk;
    Type_fr_trunk_X_fr_LF_hipassembly fr_trunk_X_fr_LF_hipassembly;
    Type_fr_LF_upperleg_X_fr_LF_hipassembly fr_LF_upperleg_X_fr_LF_hipassembly;
    Type_fr_LF_hipassembly_X_fr_LF_upperleg fr_LF_hipassembly_X_fr_LF_upperleg;
    Type_fr_LF_lowerleg_X_fr_LF_upperleg fr_LF_lowerleg_X_fr_LF_upperleg;
    Type_fr_LF_upperleg_X_fr_LF_lowerleg fr_LF_upperleg_X_fr_LF_lowerleg;
    Type_fr_RF_hipassembly_X_fr_trunk fr_RF_hipassembly_X_fr_trunk;
    Type_fr_trunk_X_fr_RF_hipassembly fr_trunk_X_fr_RF_hipassembly;
    Type_fr_RF_upperleg_X_fr_RF_hipassembly fr_RF_upperleg_X_fr_RF_hipassembly;
    Type_fr_RF_hipassembly_X_fr_RF_upperleg fr_RF_hipassembly_X_fr_RF_upperleg;
    Type_fr_RF_lowerleg_X_fr_RF_upperleg fr_RF_lowerleg_X_fr_RF_upperleg;
    Type_fr_RF_upperleg_X_fr_RF_lowerleg fr_RF_upperleg_X_fr_RF_lowerleg;
    Type_fr_LH_hipassembly_X_fr_trunk fr_LH_hipassembly_X_fr_trunk;
    Type_fr_trunk_X_fr_LH_hipassembly fr_trunk_X_fr_LH_hipassembly;
    Type_fr_LH_upperleg_X_fr_LH_hipassembly fr_LH_upperleg_X_fr_LH_hipassembly;
    Type_fr_LH_hipassembly_X_fr_LH_upperleg fr_LH_hipassembly_X_fr_LH_upperleg;
    Type_fr_LH_lowerleg_X_fr_LH_upperleg fr_LH_lowerleg_X_fr_LH_upperleg;
    Type_fr_LH_upperleg_X_fr_LH_lowerleg fr_LH_upperleg_X_fr_LH_lowerleg;
    Type_fr_RH_hipassembly_X_fr_trunk fr_RH_hipassembly_X_fr_trunk;
    Type_fr_trunk_X_fr_RH_hipassembly fr_trunk_X_fr_RH_hipassembly;
    Type_fr_RH_upperleg_X_fr_RH_hipassembly fr_RH_upperleg_X_fr_RH_hipassembly;
    Type_fr_RH_hipassembly_X_fr_RH_upperleg fr_RH_hipassembly_X_fr_RH_upperleg;
    Type_fr_RH_lowerleg_X_fr_RH_upperleg fr_RH_lowerleg_X_fr_RH_upperleg;
    Type_fr_RH_upperleg_X_fr_RH_lowerleg fr_RH_upperleg_X_fr_RH_lowerleg;

protected:
    Parameters params;

}; //class 'ForceTransforms'

/**
 * The class with the homogeneous (4x4) coordinates transformation
 * matrices.
 */
class HomogeneousTransforms
{
public:
    class Dummy {};
    typedef TransformHomogeneous<Dummy>::MatrixType MatrixType;

    struct Type_fr_LF_hipassembly_X_fr_trunk : public TransformHomogeneous<Type_fr_LF_hipassembly_X_fr_trunk>
    {
        Type_fr_LF_hipassembly_X_fr_trunk();
        const Type_fr_LF_hipassembly_X_fr_trunk& update(const state_t&);
    };
    
    struct Type_fr_trunk_X_fr_LF_hipassembly : public TransformHomogeneous<Type_fr_trunk_X_fr_LF_hipassembly>
    {
        Type_fr_trunk_X_fr_LF_hipassembly();
        const Type_fr_trunk_X_fr_LF_hipassembly& update(const state_t&);
    };
    
    struct Type_fr_LF_upperleg_X_fr_LF_hipassembly : public TransformHomogeneous<Type_fr_LF_upperleg_X_fr_LF_hipassembly>
    {
        Type_fr_LF_upperleg_X_fr_LF_hipassembly();
        const Type_fr_LF_upperleg_X_fr_LF_hipassembly& update(const state_t&);
    };
    
    struct Type_fr_LF_hipassembly_X_fr_LF_upperleg : public TransformHomogeneous<Type_fr_LF_hipassembly_X_fr_LF_upperleg>
    {
        Type_fr_LF_hipassembly_X_fr_LF_upperleg();
        const Type_fr_LF_hipassembly_X_fr_LF_upperleg& update(const state_t&);
    };
    
    struct Type_fr_LF_lowerleg_X_fr_LF_upperleg : public TransformHomogeneous<Type_fr_LF_lowerleg_X_fr_LF_upperleg>
    {
        Type_fr_LF_lowerleg_X_fr_LF_upperleg();
        const Type_fr_LF_lowerleg_X_fr_LF_upperleg& update(const state_t&);
    };
    
    struct Type_fr_LF_upperleg_X_fr_LF_lowerleg : public TransformHomogeneous<Type_fr_LF_upperleg_X_fr_LF_lowerleg>
    {
        Type_fr_LF_upperleg_X_fr_LF_lowerleg();
        const Type_fr_LF_upperleg_X_fr_LF_lowerleg& update(const state_t&);
    };
    
    struct Type_fr_RF_hipassembly_X_fr_trunk : public TransformHomogeneous<Type_fr_RF_hipassembly_X_fr_trunk>
    {
        Type_fr_RF_hipassembly_X_fr_trunk();
        const Type_fr_RF_hipassembly_X_fr_trunk& update(const state_t&);
    };
    
    struct Type_fr_trunk_X_fr_RF_hipassembly : public TransformHomogeneous<Type_fr_trunk_X_fr_RF_hipassembly>
    {
        Type_fr_trunk_X_fr_RF_hipassembly();
        const Type_fr_trunk_X_fr_RF_hipassembly& update(const state_t&);
    };
    
    struct Type_fr_RF_upperleg_X_fr_RF_hipassembly : public TransformHomogeneous<Type_fr_RF_upperleg_X_fr_RF_hipassembly>
    {
        Type_fr_RF_upperleg_X_fr_RF_hipassembly();
        const Type_fr_RF_upperleg_X_fr_RF_hipassembly& update(const state_t&);
    };
    
    struct Type_fr_RF_hipassembly_X_fr_RF_upperleg : public TransformHomogeneous<Type_fr_RF_hipassembly_X_fr_RF_upperleg>
    {
        Type_fr_RF_hipassembly_X_fr_RF_upperleg();
        const Type_fr_RF_hipassembly_X_fr_RF_upperleg& update(const state_t&);
    };
    
    struct Type_fr_RF_lowerleg_X_fr_RF_upperleg : public TransformHomogeneous<Type_fr_RF_lowerleg_X_fr_RF_upperleg>
    {
        Type_fr_RF_lowerleg_X_fr_RF_upperleg();
        const Type_fr_RF_lowerleg_X_fr_RF_upperleg& update(const state_t&);
    };
    
    struct Type_fr_RF_upperleg_X_fr_RF_lowerleg : public TransformHomogeneous<Type_fr_RF_upperleg_X_fr_RF_lowerleg>
    {
        Type_fr_RF_upperleg_X_fr_RF_lowerleg();
        const Type_fr_RF_upperleg_X_fr_RF_lowerleg& update(const state_t&);
    };
    
    struct Type_fr_LH_hipassembly_X_fr_trunk : public TransformHomogeneous<Type_fr_LH_hipassembly_X_fr_trunk>
    {
        Type_fr_LH_hipassembly_X_fr_trunk();
        const Type_fr_LH_hipassembly_X_fr_trunk& update(const state_t&);
    };
    
    struct Type_fr_trunk_X_fr_LH_hipassembly : public TransformHomogeneous<Type_fr_trunk_X_fr_LH_hipassembly>
    {
        Type_fr_trunk_X_fr_LH_hipassembly();
        const Type_fr_trunk_X_fr_LH_hipassembly& update(const state_t&);
    };
    
    struct Type_fr_LH_upperleg_X_fr_LH_hipassembly : public TransformHomogeneous<Type_fr_LH_upperleg_X_fr_LH_hipassembly>
    {
        Type_fr_LH_upperleg_X_fr_LH_hipassembly();
        const Type_fr_LH_upperleg_X_fr_LH_hipassembly& update(const state_t&);
    };
    
    struct Type_fr_LH_hipassembly_X_fr_LH_upperleg : public TransformHomogeneous<Type_fr_LH_hipassembly_X_fr_LH_upperleg>
    {
        Type_fr_LH_hipassembly_X_fr_LH_upperleg();
        const Type_fr_LH_hipassembly_X_fr_LH_upperleg& update(const state_t&);
    };
    
    struct Type_fr_LH_lowerleg_X_fr_LH_upperleg : public TransformHomogeneous<Type_fr_LH_lowerleg_X_fr_LH_upperleg>
    {
        Type_fr_LH_lowerleg_X_fr_LH_upperleg();
        const Type_fr_LH_lowerleg_X_fr_LH_upperleg& update(const state_t&);
    };
    
    struct Type_fr_LH_upperleg_X_fr_LH_lowerleg : public TransformHomogeneous<Type_fr_LH_upperleg_X_fr_LH_lowerleg>
    {
        Type_fr_LH_upperleg_X_fr_LH_lowerleg();
        const Type_fr_LH_upperleg_X_fr_LH_lowerleg& update(const state_t&);
    };
    
    struct Type_fr_RH_hipassembly_X_fr_trunk : public TransformHomogeneous<Type_fr_RH_hipassembly_X_fr_trunk>
    {
        Type_fr_RH_hipassembly_X_fr_trunk();
        const Type_fr_RH_hipassembly_X_fr_trunk& update(const state_t&);
    };
    
    struct Type_fr_trunk_X_fr_RH_hipassembly : public TransformHomogeneous<Type_fr_trunk_X_fr_RH_hipassembly>
    {
        Type_fr_trunk_X_fr_RH_hipassembly();
        const Type_fr_trunk_X_fr_RH_hipassembly& update(const state_t&);
    };
    
    struct Type_fr_RH_upperleg_X_fr_RH_hipassembly : public TransformHomogeneous<Type_fr_RH_upperleg_X_fr_RH_hipassembly>
    {
        Type_fr_RH_upperleg_X_fr_RH_hipassembly();
        const Type_fr_RH_upperleg_X_fr_RH_hipassembly& update(const state_t&);
    };
    
    struct Type_fr_RH_hipassembly_X_fr_RH_upperleg : public TransformHomogeneous<Type_fr_RH_hipassembly_X_fr_RH_upperleg>
    {
        Type_fr_RH_hipassembly_X_fr_RH_upperleg();
        const Type_fr_RH_hipassembly_X_fr_RH_upperleg& update(const state_t&);
    };
    
    struct Type_fr_RH_lowerleg_X_fr_RH_upperleg : public TransformHomogeneous<Type_fr_RH_lowerleg_X_fr_RH_upperleg>
    {
        Type_fr_RH_lowerleg_X_fr_RH_upperleg();
        const Type_fr_RH_lowerleg_X_fr_RH_upperleg& update(const state_t&);
    };
    
    struct Type_fr_RH_upperleg_X_fr_RH_lowerleg : public TransformHomogeneous<Type_fr_RH_upperleg_X_fr_RH_lowerleg>
    {
        Type_fr_RH_upperleg_X_fr_RH_lowerleg();
        const Type_fr_RH_upperleg_X_fr_RH_lowerleg& update(const state_t&);
    };
    
public:
    HomogeneousTransforms();
    void updateParams(const Params_lengths&, const Params_angles&);

    Type_fr_LF_hipassembly_X_fr_trunk fr_LF_hipassembly_X_fr_trunk;
    Type_fr_trunk_X_fr_LF_hipassembly fr_trunk_X_fr_LF_hipassembly;
    Type_fr_LF_upperleg_X_fr_LF_hipassembly fr_LF_upperleg_X_fr_LF_hipassembly;
    Type_fr_LF_hipassembly_X_fr_LF_upperleg fr_LF_hipassembly_X_fr_LF_upperleg;
    Type_fr_LF_lowerleg_X_fr_LF_upperleg fr_LF_lowerleg_X_fr_LF_upperleg;
    Type_fr_LF_upperleg_X_fr_LF_lowerleg fr_LF_upperleg_X_fr_LF_lowerleg;
    Type_fr_RF_hipassembly_X_fr_trunk fr_RF_hipassembly_X_fr_trunk;
    Type_fr_trunk_X_fr_RF_hipassembly fr_trunk_X_fr_RF_hipassembly;
    Type_fr_RF_upperleg_X_fr_RF_hipassembly fr_RF_upperleg_X_fr_RF_hipassembly;
    Type_fr_RF_hipassembly_X_fr_RF_upperleg fr_RF_hipassembly_X_fr_RF_upperleg;
    Type_fr_RF_lowerleg_X_fr_RF_upperleg fr_RF_lowerleg_X_fr_RF_upperleg;
    Type_fr_RF_upperleg_X_fr_RF_lowerleg fr_RF_upperleg_X_fr_RF_lowerleg;
    Type_fr_LH_hipassembly_X_fr_trunk fr_LH_hipassembly_X_fr_trunk;
    Type_fr_trunk_X_fr_LH_hipassembly fr_trunk_X_fr_LH_hipassembly;
    Type_fr_LH_upperleg_X_fr_LH_hipassembly fr_LH_upperleg_X_fr_LH_hipassembly;
    Type_fr_LH_hipassembly_X_fr_LH_upperleg fr_LH_hipassembly_X_fr_LH_upperleg;
    Type_fr_LH_lowerleg_X_fr_LH_upperleg fr_LH_lowerleg_X_fr_LH_upperleg;
    Type_fr_LH_upperleg_X_fr_LH_lowerleg fr_LH_upperleg_X_fr_LH_lowerleg;
    Type_fr_RH_hipassembly_X_fr_trunk fr_RH_hipassembly_X_fr_trunk;
    Type_fr_trunk_X_fr_RH_hipassembly fr_trunk_X_fr_RH_hipassembly;
    Type_fr_RH_upperleg_X_fr_RH_hipassembly fr_RH_upperleg_X_fr_RH_hipassembly;
    Type_fr_RH_hipassembly_X_fr_RH_upperleg fr_RH_hipassembly_X_fr_RH_upperleg;
    Type_fr_RH_lowerleg_X_fr_RH_upperleg fr_RH_lowerleg_X_fr_RH_upperleg;
    Type_fr_RH_upperleg_X_fr_RH_lowerleg fr_RH_upperleg_X_fr_RH_lowerleg;

protected:
    Parameters params;

}; //class 'HomogeneousTransforms'

}
}

#endif
