
%% A structure with the HyQ robot model in the format described
%% in "A beginner's guide to 6-D vectors (part 2)" by Roy Featherstone.
%%
%% HYQ = HYQ(consts, params)
%%
%% HYQ is the data struct with the robot model
%% parameters.
%% Depending on the original robot model in the Kinematics-DSL format,
%% and on configuration options, this function may take up to two
%% arguments:
%%
%% CONSTS is a struct with all the constant properties
%% of the robot model; it is required if constant-folding was disabled
%% during code generation. Otherwise, the constant numerical properties
%% of the model appear explicitly as float literals.
%%
%% PARAMS is a struct with the current value of the
%% possibly varying model properties, i.e. the model parameters. This
%% argument is required if the original robot model is parametric. If
%% not, ignore the argument.
%%
%% This file has been automatically generated by RobCoGen.

function hyq = HyQ(consts, params)

hyq.robotname = 'HyQ';

hyq.NB = 13 + 5;
hyq.parent = zeros(1, 12 + 5);

hyq.jtype(1:6) = {'Px', 'Py', 'Pz', 'Rx', 'Ry', 'Rz'};
hyq.parent(1:6) = [0 1 2 3 4 5];

for i = 1:6
    % For the 'I' field, a loop until i=5 would be enough
    hyq.I{i} = mcI( 0, [0,0,0], zeros(3) );
    % 'Xtree' instead must be initialized up to i=6
    hyq.Xtree{i} = eye(6);
end

hyq.parent(1+6) = 6;
hyq.parent(2+6) = 1+6;
hyq.parent(3+6) = 2+6;
hyq.parent(4+6) = 6;
hyq.parent(5+6) = 4+6;
hyq.parent(6+6) = 5+6;
hyq.parent(7+6) = 6;
hyq.parent(8+6) = 7+6;
hyq.parent(9+6) = 8+6;
hyq.parent(10+6) = 6;
hyq.parent(11+6) = 10+6;
hyq.parent(12+6) = 11+6;

hyq.jtype{1+6} =  'Rz' ;
hyq.jtype{2+6} =  'Rz' ;
hyq.jtype{3+6} =  'Rz' ;
hyq.jtype{4+6} =  'Rz' ;
hyq.jtype{5+6} =  'Rz' ;
hyq.jtype{6+6} =  'Rz' ;
hyq.jtype{7+6} =  'Rz' ;
hyq.jtype{8+6} =  'Rz' ;
hyq.jtype{9+6} =  'Rz' ;
hyq.jtype{10+6} =  'Rz' ;
hyq.jtype{11+6} =  'Rz' ;
hyq.jtype{12+6} =  'Rz' ;

hyq.Xtree{1+6} = rotz(3.1415927410125732) * roty(-1.5707963705062866) * rotx(0.0) * ...
          xlt([consts.tx_LF_HAA consts.ty_LF_HAA 0.0]);
hyq.Xtree{2+6} = rotz(0.0) * roty(0.0) * rotx(1.5707963705062866) * ...
          xlt([consts.tx_LF_HFE 0.0 0.0]);
hyq.Xtree{3+6} = rotz(0.0) * roty(0.0) * rotx(0.0) * ...
          xlt([consts.tx_LF_KFE 0.0 0.0]);
hyq.Xtree{4+6} = rotz(0.0) * roty(1.5707963705062866) * rotx(0.0) * ...
          xlt([consts.tx_RF_HAA consts.ty_RF_HAA 0.0]);
hyq.Xtree{5+6} = rotz(0.0) * roty(0.0) * rotx(-1.5707963705062866) * ...
          xlt([consts.tx_RF_HFE 0.0 0.0]);
hyq.Xtree{6+6} = rotz(0.0) * roty(0.0) * rotx(0.0) * ...
          xlt([consts.tx_RF_KFE 0.0 0.0]);
hyq.Xtree{7+6} = rotz(3.1415927410125732) * roty(-1.5707963705062866) * rotx(0.0) * ...
          xlt([consts.tx_LH_HAA consts.ty_LH_HAA 0.0]);
hyq.Xtree{8+6} = rotz(0.0) * roty(0.0) * rotx(1.5707963705062866) * ...
          xlt([consts.tx_LH_HFE 0.0 0.0]);
hyq.Xtree{9+6} = rotz(0.0) * roty(0.0) * rotx(0.0) * ...
          xlt([consts.tx_LH_KFE 0.0 0.0]);
hyq.Xtree{10+6} = rotz(0.0) * roty(1.5707963705062866) * rotx(0.0) * ...
          xlt([consts.tx_RH_HAA consts.ty_RH_HAA 0.0]);
hyq.Xtree{11+6} = rotz(0.0) * roty(0.0) * rotx(-1.5707963705062866) * ...
          xlt([consts.tx_RH_HFE 0.0 0.0]);
hyq.Xtree{12+6} = rotz(0.0) * roty(0.0) * rotx(0.0) * ...
          xlt([consts.tx_RH_KFE 0.0 0.0]);


tmpI = ...
[[ consts.ix_trunk  ,	-(consts.ixy_trunk),	-(consts.ixz_trunk)];
[-(consts.ixy_trunk),	  consts.iy_trunk  ,	-(consts.iyz_trunk)];
[-(consts.ixz_trunk),	-(consts.iyz_trunk),	  consts.iz_trunk]];
tmpC = skew([consts.comx_trunk; consts.comy_trunk; 0.0]);
tmpM = consts.m_trunk;
hyq.I{6} = [tmpI, tmpM *tmpC; tmpM *tmpC', eye(3)*tmpM ];

tmpI = ...
[[ consts.ix_LF_hipassembly  ,	-(consts.ixy_LF_hipassembly),	-(consts.ixz_LF_hipassembly)];
[-(consts.ixy_LF_hipassembly),	  consts.iy_LF_hipassembly  ,	-(consts.iyz_LF_hipassembly)];
[-(consts.ixz_LF_hipassembly),	-(consts.iyz_LF_hipassembly),	  consts.iz_LF_hipassembly]];
tmpC = skew([consts.comx_LF_hipassembly; 0.0; consts.comz_LF_hipassembly]);
tmpM = consts.m_LF_hipassembly;
hyq.I{7} = [tmpI, tmpM *tmpC; tmpM *tmpC', eye(3)*tmpM ];


tmpI = ...
[[ consts.ix_LF_upperleg  ,	-(consts.ixy_LF_upperleg),	-(consts.ixz_LF_upperleg)];
[-(consts.ixy_LF_upperleg),	  consts.iy_LF_upperleg  ,	-(consts.iyz_LF_upperleg)];
[-(consts.ixz_LF_upperleg),	-(consts.iyz_LF_upperleg),	  consts.iz_LF_upperleg]];
tmpC = skew([consts.comx_LF_upperleg; consts.comy_LF_upperleg; 0.0]);
tmpM = consts.m_LF_upperleg;
hyq.I{8} = [tmpI, tmpM *tmpC; tmpM *tmpC', eye(3)*tmpM ];


tmpI = ...
[[ consts.ix_LF_lowerleg  ,	-(0.0),	-(0.0)];
[-(0.0),	  consts.iy_LF_lowerleg  ,	-(0.0)];
[-(0.0),	-(0.0),	  consts.iz_LF_lowerleg]];
tmpC = skew([consts.comx_LF_lowerleg; consts.comy_LF_lowerleg; consts.comz_LF_lowerleg]);
tmpM = consts.m_LF_lowerleg;
hyq.I{9} = [tmpI, tmpM *tmpC; tmpM *tmpC', eye(3)*tmpM ];


tmpI = ...
[[ consts.ix_RF_hipassembly  ,	-(consts.ixy_RF_hipassembly),	-(consts.ixz_RF_hipassembly)];
[-(consts.ixy_RF_hipassembly),	  consts.iy_RF_hipassembly  ,	-(consts.iyz_RF_hipassembly)];
[-(consts.ixz_RF_hipassembly),	-(consts.iyz_RF_hipassembly),	  consts.iz_RF_hipassembly]];
tmpC = skew([consts.comx_RF_hipassembly; 0.0; consts.comz_RF_hipassembly]);
tmpM = consts.m_RF_hipassembly;
hyq.I{10} = [tmpI, tmpM *tmpC; tmpM *tmpC', eye(3)*tmpM ];


tmpI = ...
[[ consts.ix_RF_upperleg  ,	-(consts.ixy_RF_upperleg),	-(consts.ixz_RF_upperleg)];
[-(consts.ixy_RF_upperleg),	  consts.iy_RF_upperleg  ,	-(consts.iyz_RF_upperleg)];
[-(consts.ixz_RF_upperleg),	-(consts.iyz_RF_upperleg),	  consts.iz_RF_upperleg]];
tmpC = skew([consts.comx_RF_upperleg; consts.comy_RF_upperleg; 0.0]);
tmpM = consts.m_RF_upperleg;
hyq.I{11} = [tmpI, tmpM *tmpC; tmpM *tmpC', eye(3)*tmpM ];


tmpI = ...
[[ consts.ix_RF_lowerleg  ,	-(0.0),	-(0.0)];
[-(0.0),	  consts.iy_RF_lowerleg  ,	-(0.0)];
[-(0.0),	-(0.0),	  consts.iz_RF_lowerleg]];
tmpC = skew([consts.comx_RF_lowerleg; consts.comy_RF_lowerleg; consts.comz_RF_lowerleg]);
tmpM = consts.m_RF_lowerleg;
hyq.I{12} = [tmpI, tmpM *tmpC; tmpM *tmpC', eye(3)*tmpM ];


tmpI = ...
[[ consts.ix_LH_hipassembly  ,	-(consts.ixy_LH_hipassembly),	-(consts.ixz_LH_hipassembly)];
[-(consts.ixy_LH_hipassembly),	  consts.iy_LH_hipassembly  ,	-(consts.iyz_LH_hipassembly)];
[-(consts.ixz_LH_hipassembly),	-(consts.iyz_LH_hipassembly),	  consts.iz_LH_hipassembly]];
tmpC = skew([consts.comx_LH_hipassembly; 0.0; consts.comz_LH_hipassembly]);
tmpM = consts.m_LH_hipassembly;
hyq.I{13} = [tmpI, tmpM *tmpC; tmpM *tmpC', eye(3)*tmpM ];


tmpI = ...
[[ consts.ix_LH_upperleg  ,	-(consts.ixy_LH_upperleg),	-(consts.ixz_LH_upperleg)];
[-(consts.ixy_LH_upperleg),	  consts.iy_LH_upperleg  ,	-(consts.iyz_LH_upperleg)];
[-(consts.ixz_LH_upperleg),	-(consts.iyz_LH_upperleg),	  consts.iz_LH_upperleg]];
tmpC = skew([consts.comx_LH_upperleg; consts.comy_LH_upperleg; 0.0]);
tmpM = consts.m_LH_upperleg;
hyq.I{14} = [tmpI, tmpM *tmpC; tmpM *tmpC', eye(3)*tmpM ];


tmpI = ...
[[ consts.ix_LH_lowerleg  ,	-(0.0),	-(0.0)];
[-(0.0),	  consts.iy_LH_lowerleg  ,	-(0.0)];
[-(0.0),	-(0.0),	  consts.iz_LH_lowerleg]];
tmpC = skew([consts.comx_LH_lowerleg; consts.comy_LH_lowerleg; consts.comz_LH_lowerleg]);
tmpM = consts.m_LH_lowerleg;
hyq.I{15} = [tmpI, tmpM *tmpC; tmpM *tmpC', eye(3)*tmpM ];


tmpI = ...
[[ consts.ix_RH_hipassembly  ,	-(consts.ixy_RH_hipassembly),	-(consts.ixz_RH_hipassembly)];
[-(consts.ixy_RH_hipassembly),	  consts.iy_RH_hipassembly  ,	-(consts.iyz_RH_hipassembly)];
[-(consts.ixz_RH_hipassembly),	-(consts.iyz_RH_hipassembly),	  consts.iz_RH_hipassembly]];
tmpC = skew([consts.comx_RH_hipassembly; 0.0; consts.comz_RH_hipassembly]);
tmpM = consts.m_RH_hipassembly;
hyq.I{16} = [tmpI, tmpM *tmpC; tmpM *tmpC', eye(3)*tmpM ];


tmpI = ...
[[ consts.ix_RH_upperleg  ,	-(consts.ixy_RH_upperleg),	-(consts.ixz_RH_upperleg)];
[-(consts.ixy_RH_upperleg),	  consts.iy_RH_upperleg  ,	-(consts.iyz_RH_upperleg)];
[-(consts.ixz_RH_upperleg),	-(consts.iyz_RH_upperleg),	  consts.iz_RH_upperleg]];
tmpC = skew([consts.comx_RH_upperleg; consts.comy_RH_upperleg; 0.0]);
tmpM = consts.m_RH_upperleg;
hyq.I{17} = [tmpI, tmpM *tmpC; tmpM *tmpC', eye(3)*tmpM ];


tmpI = ...
[[ consts.ix_RH_lowerleg  ,	-(0.0),	-(0.0)];
[-(0.0),	  consts.iy_RH_lowerleg  ,	-(0.0)];
[-(0.0),	-(0.0),	  consts.iz_RH_lowerleg]];
tmpC = skew([consts.comx_RH_lowerleg; consts.comy_RH_lowerleg; consts.comz_RH_lowerleg]);
tmpM = consts.m_RH_lowerleg;
hyq.I{18} = [tmpI, tmpM *tmpC; tmpM *tmpC', eye(3)*tmpM ];

